/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.geometry;

/**
 * Same as: 3D rectangle, rectangular box, rectangular parallelopiped, cuboid,
 * rhumboid, hexahedron, rectangular prism.
 */
public class Box implements Cloneable {

    public final Point3D p1;
    public final Point3D p2;

    public Box() {
        p1 = new Point3D();
        p2 = new Point3D();
    }

    public Box(final Point3D p1, final Point3D p2) {
        this.p1 = p1;
        this.p2 = p2;
    }

    public Box addBorder(final double border) {

        if (p1.x < p2.x) {
            p1.translateX(-border);
            p2.translateX(border);
        } else {
            p1.translateX(border);
            p2.translateX(-border);
        }

        if (p1.y < p2.y) {
            p1.translateY(-border);
            p2.translateY(border);
        } else {
            p1.translateY(border);
            p2.translateY(-border);
        }

        if (p1.z < p2.z) {
            p1.translateZ(-border);
            p2.translateZ(border);
        } else {
            p1.translateZ(border);
            p2.translateZ(-border);
        }

        return this;
    }

    @Override
    public Box clone() {
        return new Box(p1.clone(), p2.clone());
    }

    public double getDepth() {
        return Math.abs(p1.z - p2.z);
    }

    public double getHeight() {
        return Math.abs(p1.y - p2.y);
    }

    public double getWidth() {
        return Math.abs(p1.x - p2.x);
    }

    public void setSizeCentered(final Point3D size) {
        p2.clone(size).scaleDown(2);
        p1.clone(p2).invert();
    }

}
