/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.geometry;

import static java.lang.Math.sqrt;

public class Point2D implements Cloneable {

    public double x, y;

    public Point2D() {
    }

    public Point2D(final double x, final double y) {
        this.x = x;
        this.y = y;
    }

    public Point2D(final Point2D parent) {
        x = parent.x;
        y = parent.y;
    }

    /**
     * Add other point to current point.
     *
     * @return current point.
     */
    public Point2D add(final Point2D otherPoint) {
        x += otherPoint.x;
        y += otherPoint.y;
        return this;
    }

    public boolean isZero() {
        return (x == 0) && (y == 0);
    }

    @Override
    public Point2D clone() {
        return new Point2D(this);
    }

    public void clone(final Point2D parent) {
        x = parent.x;
        y = parent.y;
    }

    public Point2D getMiddle(final Point2D p1, final Point2D p2) {
        x = (p1.x + p2.x) / 2d;
        y = (p1.y + p2.y) / 2d;
        return this;
    }

    public double getAngleXY(final Point2D anotherPoint) {
        return Math.atan2(x - anotherPoint.x, y - anotherPoint.y);
    }

    /**
     * Compute distance to another point.
     */
    public double getDistanceTo(final Point2D anotherPoint) {
        final double xDiff = x - anotherPoint.x;
        final double yDiff = y - anotherPoint.y;

        return sqrt(((xDiff * xDiff) + (yDiff * yDiff)));
    }

    public double getVectorLength() {
        return sqrt(((x * x) + (y * y)));
    }

    public Point2D invert() {
        x = -x;
        y = -y;
        return this;
    }

    public void roundToInteger() {
        x = (int) x;
        y = (int) y;
    }

    /**
     * Subtract other point from current point.
     *
     * @return current point.
     */
    public Point2D subtract(final Point2D otherPoint) {
        x -= otherPoint.x;
        y -= otherPoint.y;
        return this;
    }

    public Point3D to3D() {
        return new Point3D(x, y, 0);
    }

    public Point2D zero() {
        x = 0;
        y = 0;
        return this;
    }

    @Override
    public String toString() {
        return "Point2D{" +
                "x=" + x +
                ", y=" + y +
                '}';
    }
}
