/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.gui;

/**
 * A pointer to a character in a text.
 */
public class TextPointer implements Comparable<TextPointer> {

    public int row;
    public int column;

    public TextPointer() {
        this(0, 0);
    }

    public TextPointer(final int row, final int column) {
        this.row = row;
        this.column = column;
    }

    public TextPointer(final TextPointer parent) {
        this(parent.row, parent.column);
    }

    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;

        return o instanceof TextPointer && compareTo((TextPointer) o) == 0;
    }

    @Override
    public int hashCode() {
        int result = row;
        result = 31 * result + column;
        return result;
    }

    @Override
    public int compareTo(final TextPointer textPointer) {

        if (textPointer.row > row)
            return -1;
        if (textPointer.row < row)
            return 1;

        return Integer.compare(column, textPointer.column);

    }

    /**
     * Checks if this pointer is between the specified pointers.
     *
     * @param start
     *            The start pointer.
     * @param end
     *            The end pointer.
     * @return True if this pointer is between the specified pointers.
     */
    public boolean isBetween(final TextPointer start, final TextPointer end) {

        if (start == null)
            return false;

        if (end == null)
            return false;

        // Make sure that start is smaller than end.
        TextPointer smaller;
        TextPointer bigger;

        if (end.compareTo(start) >= 0) {
            smaller = start;
            bigger = end;
        } else {
            smaller = end;
            bigger = start;
        }

        // Check if this pointer is between the specified pointers.
        return (compareTo(smaller) >= 0) && (bigger.compareTo(this) > 0);
    }

}
