/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.gui;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.math.Vertex;
import eu.svjatoslav.sixth.e3d.math.TransformsPipeline;

public class UserRelativityTracker {

    private final static int minimalSliceFactor = 5;
    public Vertex center = new Vertex();
    public Vertex right;
    public Vertex down;

    public UserRelativityTracker() {

    }

    public void analyze(final TransformsPipeline transformPipe,
                        final RenderingContext renderingContext) {

        center.transform(transformPipe, renderingContext);

        if (right != null) {
            right.transform(transformPipe, renderingContext);
            down.transform(transformPipe, renderingContext);
        }
    }

    public void enableOrientationTracking() {
        right = new Vertex(new Point3D(10, 0, 0));
        down = new Vertex(new Point3D(0, 10, 0));
    }

    public double getAngleXY() {
        return center.transformedCoordinate
                .getAngleXY(down.transformedCoordinate);
    }

    public double getAngleXZ() {
        return center.transformedCoordinate
                .getAngleXZ(right.transformedCoordinate);
    }

    public double getAngleYZ() {
        return center.transformedCoordinate
                .getAngleYZ(down.transformedCoordinate);
    }

    public double getDistanceToUser() {
        return center.transformedCoordinate.getVectorLength();
    }

    public double proposeSliceFactor() {
        final double distanceToCamera = getDistanceToUser();

        double proposedSliceFactor = distanceToCamera / 5;

        if (proposedSliceFactor < minimalSliceFactor)
            proposedSliceFactor = minimalSliceFactor;

        return proposedSliceFactor;
    }

}
