/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.gui;

public interface ViewRenderListener {

    /**
     * Notifies that it is about time (to keep constant framerate) to render next frame and
     * allows listener to do any related processing that it needs to.
     * <p>
     * Each {@link ViewRenderListener} will be notified exactly once before every frame is rendered.
     * <p>
     * {@link ViewRenderListener} can determine if frame repaint is actually
     * needed from its perspective. Frame will be rendered only if at least one listener says yes.
     * This mechanism allows to save computing power and energy by skipping frame rendering when possible.
     *
     * @return <code>true</code> if underlying view shall be re-rendered. If at least one of the view update listeners
     * returns <code>true</code>, view is re-rendered.
     */
    boolean beforeRender(ViewPanel viewPanel, final int millisecondsSinceLastFrame);
}
