/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.Avatar;
import eu.svjatoslav.sixth.e3d.gui.RenderingContext;
import eu.svjatoslav.sixth.e3d.gui.ViewPanel;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.math.TransformsPipeline;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.AbstractShape;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class ShapeCollection {

    private final RenderAggregator aggregator = new RenderAggregator();
    private final TransformsPipeline transformPipe = new TransformsPipeline();
    private final List<AbstractShape> shapes = new ArrayList<>();

    public synchronized void addShape(final AbstractShape shape) {
        shapes.add(shape);
    }

    public Collection<AbstractShape> getShapes() {
        return shapes;
    }

    public void clear() {
        shapes.clear();
    }

    public synchronized void paint(final ViewPanel viewPanel,
                                   final RenderingContext renderingContext) {

        renderingContext.frameNumber++;

        aggregator.reset();
        transformPipe.clear();

        // translate scene according to camera current location
        final Avatar avatar = viewPanel.getAvatar();

        // rotate scene according to avatar looking direction
        transformPipe.addTransform(new Transform(new Point3D(), avatar
                .getAngleXZ(), avatar.getAngleYZ()));

        // translate scene according to avatar location in the world
        final Point3D translation = new Point3D(-avatar.getLocation().x,
                -avatar.getLocation().y, -avatar.getLocation().z);

        transformPipe.addTransform(new Transform(translation, 0, 0));

        for (final AbstractShape shape : shapes)
            shape.transform(transformPipe, aggregator, renderingContext);

        aggregator.paint(renderingContext);
    }
}
