/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.wireframe;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.geometry.Rectangle;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;

public class Grid2D extends AbstractCompositeShape {

    /**
     * @param transform Grid location
     * @param rectangle Grid dimensions
     * @param xDivisionCount Division count along X axis
     * @param yDivisionCount Division count along Y axis
     * @param appearance Grid lines appearance
     */
    public Grid2D(final Transform transform, final Rectangle rectangle,
                  final int xDivisionCount, final int yDivisionCount,
                  final LineAppearance appearance) {

        super(transform);

        final double stepY = rectangle.getHeight() / yDivisionCount;
        final double stepX = rectangle.getWidth() / xDivisionCount;

        for (int ySlice = 0; ySlice <= yDivisionCount; ySlice++) {
            final double y = (ySlice * stepY) + rectangle.getLowerY();

            for (int xSlice = 0; xSlice <= xDivisionCount; xSlice++) {
                final double x = (xSlice * stepX) + rectangle.getLowerX();

                final Point3D p1 = new Point3D(x, y, 0);
                final Point3D p2 = new Point3D(x + stepX, y, 0);
                final Point3D p3 = new Point3D(x, y + stepY, 0);

                if (xSlice < xDivisionCount)
                    addShape(appearance.getLine(p1, p2));

                if (ySlice < yDivisionCount)
                    addShape(appearance.getLine(p1, p3));
            }

        }

    }

}
